/******************************************************************
SQL Saturday #326: October 25th, Tampa, FL.

"Clustered Columnstore Indexes: Internals and Design Considerations"
Dmitri V. Korotkevitch

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

01. Compression and Size
******************************************************************/
use SQLSat326
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'DemoSize_Heap'
)
	drop table dbo.DemoSize_Heap
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'DemoSize_Row'
)
	drop table dbo.DemoSize_Row
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'DemoSize_Page'
)
	drop table dbo.DemoSize_Page
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'DemoSize_ColumnStore'
)
	drop table dbo.DemoSize_ColumnStore
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'DemoSize_ColumnStoreArchival'
)
	drop table dbo.DemoSize_ColumnStoreArchival
go

create table dbo.DemoSize_Heap
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

create table dbo.DemoSize_Row
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

create table dbo.DemoSize_Page
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

create table dbo.DemoSize_Columnstore
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

create table dbo.DemoSize_ColumnstoreArchival
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

-- Run: cci_bcp.cmd DemoSize_Heap CCIMed.txt 1000000

insert into dbo.DemoSize_Page
	select * from dbo.DemoSize_Heap
go

alter table [dbo].[DemoSize_Page] rebuild with (data_compression=page)
go

insert into dbo.DemoSize_Row
	select * from dbo.DemoSize_Page
go

alter table [dbo].[DemoSize_Row] rebuild with (data_compression=row)
go

insert into dbo.DemoSize_Columnstore
	select * from dbo.DemoSize_Page
go

create clustered columnstore index CCI_DemoSize_Columnstore
on dbo.DemoSize_Columnstore
with (data_compression=columnstore)
go

insert into dbo.DemoSize_ColumnstoreArchival
	select * from dbo.DemoSize_Columnstore
go

create clustered columnstore index CCI_DemoSize_ColumnstoreArchive
on dbo.DemoSize_ColumnstoreArchival
with (data_compression=columnstore_archive)
go

exec sp_spaceused 'dbo.DemoSize_Heap'
exec sp_spaceused 'dbo.DemoSize_Row'
exec sp_spaceused 'dbo.DemoSize_Page'
exec sp_spaceused 'dbo.DemoSize_ColumnStore'
exec sp_spaceused 'dbo.DemoSize_ColumnStoreArchival'
go

