/******************************************************************
SQL Saturday #326: October 25th, Tampa, FL.

"Clustered Columnstore Indexes: Internals and Design Considerations"
Dmitri V. Korotkevitch

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

04. Data Import and Batch Size
******************************************************************/
use SQLSat326
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'Import_1M'
)
	drop table dbo.Import_1M
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'Import_100K'
)
	drop table dbo.Import_100K
go

if exists
(
	select *
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'Import_99K'
)
	drop table dbo.Import_99K
go

create table dbo.Import_1M
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

create clustered columnstore index CCI_Import_1M
on dbo.Import_1M
go


create table dbo.Import_100K
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

create clustered columnstore index CCI_Import_100K
on dbo.Import_100K
go


create table dbo.Import_99K
(
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[CustomerKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NOT NULL,
	[OrderQuantity] [smallint] NOT NULL,
	[UnitPrice] [money] NOT NULL,
	[ExtendedAmount] [money] NOT NULL,
	[UnitPriceDiscountPct] [float] NOT NULL,
	[DiscountAmount] [float] NOT NULL,
	[ProductStandardCost] [money] NOT NULL,
	[TotalProductCost] [money] NOT NULL,
	[SalesAmount] [money] NOT NULL,
	[TaxAmt] [money] NOT NULL,
	[Freight] [money] NOT NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL,
	[OrderDate] [datetime] NULL,
	[DueDate] [datetime] NULL,
	[ShipDate] [datetime] NULL
)
go

create clustered columnstore index CCI_Import_99K
on dbo.Import_99K
go

/* Run
cci_bcp.cmd Import_1m CCIMed.txt 1048576
cci_bcp.cmd Import_100K CCIMed.txt 102500
-- Using 
cci_bcp.cmd Import_99K CCISmall.txt 99999
*/


select * 
from sys.column_store_row_groups g 
where g.object_id = object_id(N'dbo.Import_1M')


select * 
from sys.column_store_row_groups g 
where g.object_id = object_id(N'dbo.Import_100K')
go

select * 
from sys.column_store_row_groups g 
where g.object_id = object_id(N'dbo.Import_99K')
go

-- Large Number of Row Groups

set statistics time on
go

dbcc dropcleanbuffers
go

select 
	max(ProductKey),max(OrderDateKey),max(DueDateKey)
	,max(ShipDateKey),max(CustomerKey),max(PromotionKey)
	,max(CurrencyKey),max(SalesTerritoryKey),max(SalesOrderLineNumber)
	,max(RevisionNumber),max(OrderQuantity),max(UnitPrice)
	,max(ExtendedAmount),max(UnitPriceDiscountPct),max(DiscountAmount)
	,max(ProductStandardCost),max(TotalProductCost),max(SalesAmount)
	,max(TaxAmt),max(Freight) 
from dbo.Import_1M
go

dbcc dropcleanbuffers
go

select 
	max(ProductKey),max(OrderDateKey),max(DueDateKey)
	,max(ShipDateKey),max(CustomerKey),max(PromotionKey)
	,max(CurrencyKey),max(SalesTerritoryKey),max(SalesOrderLineNumber)
	,max(RevisionNumber),max(OrderQuantity),max(UnitPrice)
	,max(ExtendedAmount),max(UnitPriceDiscountPct),max(DiscountAmount)
	,max(ProductStandardCost),max(TotalProductCost),max(SalesAmount)
	,max(TaxAmt),max(Freight) 
from dbo.Import_100K
go

set statistics time off
go

-- Large Delta Stores
select * 
from sys.column_store_row_groups g 
where g.object_id = object_id(N'dbo.Import_1M')

select * 
from sys.column_store_row_groups g 
where g.object_id = object_id(N'dbo.Import_99K')
go


set statistics time on
go

dbcc dropcleanbuffers
go

select 
	max(ProductKey),max(OrderDateKey),max(DueDateKey)
	,max(ShipDateKey),max(CustomerKey),max(PromotionKey)
	,max(CurrencyKey),max(SalesTerritoryKey),max(SalesOrderLineNumber)
	,max(RevisionNumber),max(OrderQuantity),max(UnitPrice)
	,max(ExtendedAmount),max(UnitPriceDiscountPct),max(DiscountAmount)
	,max(ProductStandardCost),max(TotalProductCost),max(SalesAmount)
	,max(TaxAmt),max(Freight) 
from dbo.Import_1M
go

dbcc dropcleanbuffers
go

select 
	max(ProductKey),max(OrderDateKey),max(DueDateKey)
	,max(ShipDateKey),max(CustomerKey),max(PromotionKey)
	,max(CurrencyKey),max(SalesTerritoryKey),max(SalesOrderLineNumber)
	,max(RevisionNumber),max(OrderQuantity),max(UnitPrice)
	,max(ExtendedAmount),max(UnitPriceDiscountPct),max(DiscountAmount)
	,max(ProductStandardCost),max(TotalProductCost),max(SalesAmount)
	,max(TaxAmt),max(Freight) 
from dbo.Import_99K
go

set statistics time off
go


-- Large Delete Bitmap
;with CTE
as
(
	select *, ROW_NUMBER() over(order by (select null)) as RowNum
	from dbo.Import_1M
)
delete from CTE where RowNum % 2 = 1
go

select * 
from sys.column_store_row_groups g 
where g.object_id = object_id(N'dbo.Import_1M')
go

set statistics time on
go

dbcc dropcleanbuffers
go

select 
	max(ProductKey),max(OrderDateKey),max(DueDateKey)
	,max(ShipDateKey),max(CustomerKey),max(PromotionKey)
	,max(CurrencyKey),max(SalesTerritoryKey),max(SalesOrderLineNumber)
	,max(RevisionNumber),max(OrderQuantity),max(UnitPrice)
	,max(ExtendedAmount),max(UnitPriceDiscountPct),max(DiscountAmount)
	,max(ProductStandardCost),max(TotalProductCost),max(SalesAmount)
	,max(TaxAmt),max(Freight) 
from dbo.Import_1M
go

dbcc dropcleanbuffers
go

select 
	max(ProductKey),max(OrderDateKey),max(DueDateKey)
	,max(ShipDateKey),max(CustomerKey),max(PromotionKey)
	,max(CurrencyKey),max(SalesTerritoryKey),max(SalesOrderLineNumber)
	,max(RevisionNumber),max(OrderQuantity),max(UnitPrice)
	,max(ExtendedAmount),max(UnitPriceDiscountPct),max(DiscountAmount)
	,max(ProductStandardCost),max(TotalProductCost),max(SalesAmount)
	,max(TaxAmt),max(Freight) 
from dbo.Import_100K
go

set statistics time off
go
